/*70-762 Lesson 1.11 Lab 4 Reviewing Data Types and Their Applications */


USE WideWorldImporters
GO

DROP SEQUENCE IF EXISTS CountBy3;
GO

/*
>>>>>>>>>>>>>>>>>>Using SEQUENCE>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
A Sequence is preferred over an Identity property when:
A number needed before data inserted into the table
Single series of numbers needs to be shared between multiple tables or multiple columns within a table.
Series needs to be recycled when a specified number is reached
*/
CREATE SEQUENCE CountBy3
   AS tinyint
    START WITH 1
    INCREMENT BY 1
    MINVALUE 1
    MAXVALUE 3
    CYCLE ;
GO

SELECT NEXT VALUE FOR CountBy3 AS SurveyGroup, Name FROM sys.objects ;
GO

/*
>>>>>>>>>>>>>>>>>>>Creating a SEQUENCE used in a table>>>>>>>>>>
*/
USE WideWorldImporters
GO

--Create the schema if not already created--
DROP SCHEMA IF EXISTS Test;
GO

CREATE SCHEMA Test;
GO

-- Create a table
CREATE TABLE Test.Orders
    (OrderID int PRIMARY KEY,
    Name varchar(20) NOT NULL,
    Qty int NOT NULL);
GO

-- Create a sequence
CREATE SEQUENCE Test.CountBy1
    START WITH 1
    INCREMENT BY 1 ;
GO

-- Insert three records
INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'lions', 2) ;
INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'tigers', 4) ;
INSERT Test.Orders (OrderID, Name, Qty)
    VALUES (NEXT VALUE FOR Test.CountBy1, 'bears', 3) ;
GO

-- View the table
SELECT * FROM Test.Orders ;
GO

--CLEANUP--
--Drop table Orders and drop schema Test --
DROP TABLE IF EXISTS Test.Orders;
GO
DROP SEQUENCE IF EXISTS Test.CountBy1
GO
--DROP SCHEMA IF EXISTS Test
--GO
-----------------------------------------------------------------
----------------------------------------------------------------
-- Creating a table using NEWID as the default value -----------
-- for the uniqueidentifier data type --------------------------
DROP TABLE IF EXISTS Test.Customers;
GO

CREATE TABLE Test.Customers
(
 CustomerID uniqueidentifier NOT NULL
   DEFAULT NEWID(),
 Company varchar(30) NOT NULL,
 ContactName varchar(60) NOT NULL, 
 CountryRegion varchar(20) NOT NULL, 
);
GO

--------------------------------------------------------------------------
--Assign the default value of NEWID() to each new and existing row -------
--This ensures a unique value for the CustomerID column ------------------
INSERT INTO Test.Customers (CustomerID,Company,ContactName,CountryRegion)
     VALUES
           (NEWID(), N'SQL Studios',N'Mickey', N'CA'),
		   (NEWID(), N'SQL Studios', N'Minnie', N'PA'),
		   (NEWID(), N'SQL Management', N'Donald', N'UK'),
		   (NEWID(), N'SQL Studios', N'Daisy', N'UK'),
		   (NEWID(), N'SQL Management', N'Grumpy', N'WA'),
		   (NEWID(), N'SQL Studios', N'Happy', N'PA'),
		   (NEWID(), N'SQL Management', N'Dopey', N'WA');
GO

SELECT *
FROM Test.Customers;
GO

DROP TABLE IF EXISTS Test.Customers;
GO

-----------------------------------------------------------------------------------
--Using Tempdb, create a table using sequential GUID and nonsequential GUID columns
------------------------------------------------------------------------------------
--Create Unique GUIDs by Using NewSequentialId() and NewId()

USE tempdb
go

---------------------------------------------------------------
--Create table for testing GUID and Identitiy column creation--
--using Identity, NEWSEQUENTIALID(), NEWID()-------------------
---------------------------------------------------------------
DROP TABLE IF EXISTS tempdb.dbo.GUIDTESTING;
GO

CREATE TABLE GUIDTESTING
(
RowNumber			int identity
,NewSequentialIDCol uniqueidentifier DEFAULT NEWSEQUENTIALID()
,NewIdCol			uniqueidentifier DEFAULT NEWID()
);
GO

--------------------------------------------------------------------
-- Insert 5 rows into the GUIDETESTING table using DEFAULT values --
-- as specified by the column datatype ----------------------------- 
DECLARE @RowCount int
SET @RowCount = 0
WHILE @RowCount < 5
BEGIN
	INSERT INTO GUIDTESTING DEFAULT VALUES
	SET @RowCount = @RowCount +1
END;
GO

-- Return the rows sorted by NewSequentialIDCol--
--Notice this sort follows the identity ---------
SELECT * FROM GUIDTESTING ORDER BY NewSequentialIDCol;
GO

-----------------------------------------------
-- Return the rows sorted by NewIDCol----------
SELECT * FROM GUIDTESTING ORDER BY NewIDCol;
GO

DROP TABLE IF EXISTS tempdb.dbo.GUIDTESTING;
GO
/*****************************************************/
/*  Use Data Types for  Dates */
----------------------------------------------------
--Querying Date and Time ---------------------------
--Know if source data includes time and if time
--other than midnight has been entered
--Otherwise queries will not resolve correctly
----------------------------------------------------

USE WideWorldImporters
GO

--Query orderdate from Sales.Orders------
SELECT OrderID, OrderDate
FROM Sales.Orders;
GO

SELECT OrderID, OrderDate
FROM Sales.Orders
WHERE OrderDate = '20130104';
GO

--------------------------------------------------------
---Create table Test.SalesOrderDateTest and fill with data
-----------------------------------------------------------

 
DROP TABLE IF EXISTS Test.SalesOrderDateTest;
GO

SELECT OrderID, OrderDate
INTO Test.SalesOrderDateTest
FROM Sales.Orders
WHERE OrderID >= 236 and OrderID < 240
GO


ALTER TABLE Test.SalesOrderDateTest
ALTER COLUMN OrderDate datetime2;
GO

SELECT * 
FROM Test.SalesOrderDateTest;
GO

-----------------------------------------------------------
--Add data with dates that have a time value also---------
-----------------------------------------------------------



INSERT INTO Test.SalesOrderDateTest(OrderID, OrderDate)
VALUES (50001, '2013-01-04 00:35'),
(50002, '2013-01-04 11:30'),
(50003, '2013-01-04 12:45');
GO




--Query the table-------------------------------------
--for all inserted orders-----------------------------
-------------------------------------------------------
SELECT OrderID, OrderDate
FROM Test.SalesOrderDateTest;
GO

----Query for a date which also has timed data-----------
----Not all values are returned--------------------------
SELECT OrderID, OrderDate
FROM Test.SalesOrderDateTest
WHERE OrderDate = '20130104'
ORDER BY OrderID;
GO

---Need to change the query to account for time past midnight
-------------------------------------------------------------
SELECT OrderID, OrderDate
FROM Test.SalesOrderDateTest
WHERE OrderDate >= '20130104' AND OrderDate < '20130105'
ORDER BY OrderID;
GO


/*-----------------------------------------------------------
---------Cleanup -DROP Table Test.SalesOrderDateTest--------

-------------------------------------------------------------  
*/
DROP TABLE IF EXISTS Test.SalesOrderDateTest;
GO

--------------------------------------------------------------
/*************************************************************/
---Using GEOGRAPHY Data Type----------------------------------
----with .Lat and .Long methods--------------------------------
/*************************************************************/
USE WideWorldImporters
GO

SELECT CityName, StateProvinceID, Location, Location.Lat AS Latitude, Location.Long AS Longitude
FROM Application.Cities
WHERE StateProvinceID IN (39, 50);

USE WideWorldImporters
GO

SELECT CityName, Location.Lat AS Latitude, Location.Long AS Longitude
FROM Application.Cities
WHERE CityName = 'Philadelphia' AND StateProvinceID = 39;

