/* 70-762 Lesson 13 Lab 3 - Understanding Query Plans */

--Graphical Query Plan conditions affecting performance:
--  Review query plan optimization property
--	 - Right click on SELECT and click Properties
--   - Under Query time Stats observe reason for termination
--  Identify operators utilizing a great deal of memory
--  Evaluate the cost of each operator
--  Note arrow width which signifies amount of data processed 
--  Observe warnings

-->>>>>>>>>>Example 1<<<<<<<<<<<<--
--Clustered Index Scan Operator and Clustered Index Seek Operator----
Use WideWorldImporters;
GO

SELECT s.StockItemID,
      s.StockItemName,
      s.SupplierID,
	  h.QuantityOnHand
FROM Warehouse.StockItems AS s
JOIN Warehouse.StockItemHoldings AS h
ON s.StockItemID = h.StockItemID
WHERE s.SupplierID = 12;
GO

-->>>>>>>>>>Example 2<<<<<<<<<<<<--
--Analyzing Query Operators----
USE WideWorldImporters;
GO

---------Table Scan-------------
SELECT *
FROM Warehouse.VehicleTemperatures;
GO

------Clustered Index Scan----------
SELECT *
FROM Purchasing.SupplierCategories;
GO


-----Clustered Index Seek-----------
SELECT *
FROM Purchasing.SupplierCategories
WHERE SupplierCategoryID = 3;

--Index Seek (NonClustered) and Key Lookup (Clustered)----
--Consider creating a covering index-----------------------
SELECT SupplierCategoryID,
	   SupplierCategoryName,
	   ValidFrom
FROM Purchasing.SupplierCategories
WHERE SupplierCategoryName = 'Novelty Goods Supplier';

---Sort on Non Key Column----------------
SELECT *
FROM Purchasing.Suppliers
ORDER BY BankAccountName;
GO

---Sort on Key Column--------------------
SELECT *
FROM Purchasing.Suppliers
ORDER BY SupplierID;
GO

---Hash Match (Aggregate)----------------
---Grouping rows contributes a large percentage to cost of query
---Notice the thickness of the arrows
---Consider minimizing performance impact by creating indexed view
---   to perform the aggregation or just minimizing rows to aggregate
SELECT
	YEAR(TransactionDate) AS 'Transaction Year',
	COUNT(SupplierTransactionID) AS 'Transaction Count'
FROM Purchasing.SupplierTransactions
GROUP BY YEAR(TransactionDate);
GO


---Hash Match (Inner Join)--------------
---Query uses two Hash Match (inner Join) operators
---Consider adding indexes to eliminate Hash Match operators
SELECT
	si.StockItemName,
	c.ColorName,
	s.SupplierName
FROM Warehouse.StockItems AS si
	JOIN Warehouse.Colors AS c
ON si.ColorID = c.ColorID
	JOIN Purchasing.Suppliers AS s
ON s.SupplierID = si.SupplierID;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>.