/*70-762 Lesson 3.4 Lab 2 - Analyzing the Effects of DML Statements Using a View Object */

Use WideWorldImporters;
GO

--Create Examples.ReallyGreatCustomers for scenario-------------------
DROP TABLE IF EXISTS Examples.ReallyGreatCustomers;
GO   

SELECT CustomerID, CustomerName, CustomerCategoryID, StandardDiscountPercentage 
INTO Examples.ReallyGreatCustomers
FROM Sales.Customers;
GO

SELECT *
FROM Examples.ReallyGreatCustomers;
GO

--Trial JOIN which will be used in referencing data to create view------------
SELECT rgc.CustomerID
	, rgc.CustomerName
	, rgc.CustomerCategoryID
	, cc.CustomerCategoryName
	, rgc.StandardDiscountPercentage
FROM Examples.ReallyGreatCustomers AS rgc
JOIN
Sales.CustomerCategories AS cc
ON rgc.CustomerCategoryID = cc.CustomerCategoryID;
GO


--CREATE view - drop if already created---------------
DROP VIEW IF EXISTS Examples.vReallyGreatCustomerInfo;
GO


--View includes a JOIN of Examples.ReallyGreatCustomers 
--and Sales.CustomerCategories tables
CREATE VIEW Examples.vReallyGreatCustomerInfo
	WITH SCHEMABINDING  --ensure base table structure cannot be modified
AS
SELECT 
	rgc.CustomerID
	, rgc.CustomerName
	, rgc.CustomerCategoryID
	, cc.CustomerCategoryName
	, rgc.StandardDiscountPercentage
FROM Examples.ReallyGreatCustomers AS rgc
JOIN
Sales.CustomerCategories AS cc
ON rgc.CustomerCategoryID = cc.CustomerCategoryID;
GO

SELECT *
FROM Examples.vReallyGreatCustomerInfo;
GO


--UPDATE Examples.ReallyGreatCustomers table using view----------
--Give all Corporate customers a 10 percent discount------
--(Assuming StandardDiscountPercentage is in decimal format - not percent)--
--Corporate is CustomerCategoryID 7-----------------------
--UPDATE  is on only one table------
UPDATE Examples.vReallyGreatCustomerInfo
SET StandardDiscountPercentage = 0.10
WHERE CustomerCategoryID = 7;
GO

--Display the UPDATE in the Examples.ReallyGreatCustomers table------------
SELECT CustomerID, CustomerName, CustomerCategoryID, StandardDiscountPercentage
FROM Examples.ReallyGreatCustomers
ORDER BY CustomerCategoryID DESC;
GO


--INSERT into Examples.ReallyGreatCustomers table using view----
--INSERT involves only one table--------------------------------
INSERT INTO Examples.vReallyGreatCustomerInfo 
(CustomerID,CustomerName, CustomerCategoryID, StandardDiscountPercentage)   
VALUES (5000, 'NewGreatCustomer', 5, 0.05);
GO

SELECT CustomerID, CustomerName, CustomerCategoryID, StandardDiscountPercentage
FROM Examples.ReallyGreatCustomers
WHERE CustomerCategoryID = 5;
GO

--Deleting data from a table using a view--
--View must include ONLY ONE base TABLE----

DROP VIEW IF EXISTS Examples.vDeleteFromReallyGreatCustomers;
GO

--View includes only Examples.ReallyGreatCustomers base table--
CREATE VIEW Examples.vDeleteFromReallyGreatCustomers
	WITH SCHEMABINDING
AS
SELECT 
	rgc.CustomerID
	, rgc.CustomerName	
FROM Examples.ReallyGreatCustomers AS rgc
GO

--Deleting from the base table using the view---------
DELETE 
FROM Examples.vDeleteFromReallyGreatCustomers
WHERE CustomerID = 5000;
GO

SELECT CustomerID, CustomerName, CustomerCategoryID, StandardDiscountPercentage
FROM Examples.ReallyGreatCustomers
WHERE CustomerCategoryID = 5;
GO

--CREATE view with CHECK OPTION-------------
Use WideWorldImporters;
GO

DROP TABLE IF EXISTS Examples.NewCountries;
GO

--Create and fill table for scenatio--------
SELECT CountryName, Continent
INTO
Examples.NewCountries
FROM
Application.Countries;
GO

SELECT *
FROM
Examples.NewCountries;
GO

DROP VIEW IF EXISTS Examples.vCountriesInEurope;
GO

--Create view WITH CHECK OPTION-----------
CREATE VIEW Examples.vCountriesInEurope
	WITH SCHEMABINDING
AS
SELECT CountryName, Continent
FROM Examples.NewCountries
WHERE Continent = 'Europe'
WITH CHECK OPTION;
GO


SELECT *
FROM Examples.vCountriesInEurope;
GO

--UPDATE the view to Move France to North America---
UPDATE Examples.vCountriesInEurope
SET Continent = 'North America'
WHERE CountryName = 'France';
GO 

--Msg 550, Level 16, State 1, Line 116
--The attempted insert or update failed because the target view either
-- specifies WITH CHECK OPTION or spans a view that specifies WITH CHECK OPTION
-- and one or more rows resulting from the operation did not qualify
-- under the CHECK OPTION constraint.
--The statement has been terminated.


-- Clean up code if desired:
DROP VIEW IF EXISTS Examples.vReallyGreatCustomerInfo;
DROP VIEW IF EXISTS Examples.vDeleteFromReallyGreatCustomers;
DROP TABLE IF EXISTS Examples.ReallyGreatCustomers;
GO   
DROP VIEW IF EXISTS Examples.vCountriesInEurope;
DROP TABLE IF EXISTS Examples.NewCountries;
GO

