/*70-762 Lesson 3.8 Lab 4 - Configuring and Using Indexed Views */

Use WideWorldImporters;
GO

--Set the options to support indexed views.
SET NUMERIC_ROUNDABORT OFF;
SET ANSI_PADDING, ANSI_WARNINGS, CONCAT_NULL_YIELDS_NULL, ARITHABORT,
    QUOTED_IDENTIFIER, ANSI_NULLS ON;
GO

DROP VIEW IF EXISTS Examples.vOrderRevenue;
GO

DROP TABLE IF EXISTS Examples.NewOrderLines;
GO

--Create the Examples.NewOrderlines table--------
SELECT*
INTO
Examples.NewOrderLines
FROM Sales.OrderLines;
GO

ALTER TABLE Examples.NewOrderLines ALTER COLUMN UnitPrice decimal(18,2) NOT NULL;
GO

-----CREATING THE INDEXED VIEW on the Examples.NewOrderLines table ----
--Create view with schemabinding.
DROP VIEW IF EXISTS Examples.vOrderRevenue;
GO

CREATE VIEW Examples.vOrderRevenue
WITH SCHEMABINDING
AS
    SELECT SUM(UnitPrice*Quantity*(1.00 + TaxRate/100)) AS Revenue,
         StockItemID, COUNT_BIG(*) AS Count
    FROM Examples.NewOrderLines
    GROUP BY StockItemID;
GO

SELECT *
FROM
Examples.vOrderRevenue
ORDER BY StockItemID;
GO

--Create an index on the view.
CREATE UNIQUE CLUSTERED INDEX IDX_vOrderRevenue 
    ON Examples.vOrderRevenue (StockItemID);
GO

--This query directly references the indexed view - Examples.vOrderRevenue-----------
--Use Ctrl-M to set Actual Execution Plan ON-----------------------------------

SELECT *
FROM Examples.vOrderRevenue
WHERE Count > 1000;
GO


--This query uses the indexed view - Examples.vOrderRevenue------------------
--Even though it is not directly referenced in the query---------------------
--Use Ctrl-M to set Actual Execution Plan ON---------------------------------
SELECT SUM(UnitPrice*Quantity*(1.00 + TaxRate/100)) AS Revenue,
         StockItemID, COUNT_BIG(*) AS Count
    FROM Examples.NewOrderLines
	WHERE StockItemID BETWEEN 100 AND 200
	GROUP BY StockItemID
	ORDER BY StockItemID;
GO


-------------------------------------------------------
--Clean up code if desired:
DROP VIEW IF EXISTS Examples.vOrderRevenue ;
GO

DROP TABLE IF EXISTS Examples.NewOrderLines;
GO

