/* 70-762 Lesson 7 Lab 5 - Implementing AFTER and INSTEAD OF Triggers */

USE CertLiveDB
GO

/* Delete Triggers if already created in table */
DROP TRIGGER IF Exists Production.trCategories_AfterTest;
GO

DROP TRIGGER IF EXISTS Production.trCategories_InsteadOFCategoryNameCheck;
GO

/* CREATE the AFTER Trigger to display the inserted and deleted tables
   created by the trigger
 */
CREATE TRIGGER Production.trCategories_AfterTest
ON Production.Categories
AFTER INSERT, UPDATE
AS
BEGIN
  SELECT * FROM inserted
  SELECT * FROM deleted
END;
RAISERROR ('Category is INSERTED or UPDATED', 16, 10);
GO

/* INSERT a new row INTO the Categories table.
   The inserted and deleted tables from the trigger are displayed
*/
INSERT INTO Production.Categories (CategoryName ,description)
	VALUES(N'MCSA Windows Server 2016', N'Microsoft Solutions Associate');
GO

SELECT *
FROM Production.Categories;
GO

--INSTEAD OF TRIGGER-------------------------------------
USE CertLiveDB
GO
/* Delete Trigger if already created in table */

DROP TRIGGER IF EXISTS Production.trCategories_InsteadOFCategoryNameCheck;
GO

/* CREATE INSTEAD OF Trigger */
CREATE TRIGGER Production.trCategories_InsteadOFCategoryNameCheck
ON Production.Categories
INSTEAD OF INSERT
AS
BEGIN
  SET NOCOUNT ON;            --count of number of rows affected not returned --
  IF EXISTS (SELECT COUNT(*)
        FROM Inserted AS I
        JOIN Production.Categories AS C
          ON I.CategoryName = C.CategoryName
		GROUP BY I.CategoryName
		HAVING COUNT(*) > 0 )  --check if Inserted CategoryName is already in Production.Categories --
		THROW 50000, 'Duplicate category names are not allowed', 0; --display error message --
	ELSE
	BEGIN
		INSERT Production.Categories (CategoryName, Description) -- if not duplicate INSERT the category name --
		SELECT CategoryName, Description FROM Inserted;
	END;
END;
GO

--First Display Existing Categories
SELECT *
FROM Production.Categories;
GO

/* Test the INSTEAD OF Trigger  - INSERT a category name That already exists in the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('SQL 2016 Database Administration', 'New Category Description');
GO

/* Test the INSTEAD OF Trigger - INSERT a new category name the table */
INSERT INTO Production.Categories (CategoryName,Description)
     VALUES ('SQL 2016 BI Development', 'Microsoft Solutions Associate');
GO

--Display Resulting Categories
SELECT *
FROM Production.Categories;
GO

/************************************************************/
--Cleanup by Deleting row inserted using AFTER trigger----
DELETE FROM Production.Categories
	WHERE CategoryName = 'MCSA Windows Server 2016';
GO 

--Cleanup by Deleting row inserted using INSTEAD OF trigger---
DELETE FROM Production.Categories
     WHERE CategoryName = 'SQL 2016 BI Development';
GO

/* Cleanup by deleting Triggers */
DROP TRIGGER IF Exists Production.trCategories_AfterTest;
GO

DROP TRIGGER IF EXISTS Production.trCategories_InsteadOFCategoryNameCheck;
GO


