/* Lesson 9 Lab 1 Connection 1 - Implementing Results of Concurrent Queries Using SET TRANSACTION ISOLATION LEVEL*/

--At the beginning of this lab open both files for lab 1
--Connection 1 and Connection 2 in separate query windows
--Keep only these two connections open for the lab--------
--QUERY WINDOW 1---------------
--Connection 1-----------------

-->>>>>>Create sample table for lab<<<<<<-----------
--------------------------
USE WideWorldImporters;
GO

DROP TABLE IF EXISTS Examples.IsoLevels;
GO

SELECT [OrderLineID]
      ,[OrderID]
      ,[StockItemID]
      ,[Description]
      ,[Quantity]
      ,[UnitPrice]
 INTO Examples.IsoLevels     
 FROM [WideWorldImporters].[Sales].[OrderLines]
 WHERE OrderLineID <= 30;

 SELECT *
 FROM Examples.IsoLevels;
 GO

 -----Example 1  Connection 1--------------------------
 -----READ UNCOMMITTED ISOLATION LEVEL---
 --Open a transaction and update the price of an item
 --Keep transaction open
 BEGIN TRAN;
	UPDATE Examples.IsoLevels
	SET UnitPrice +=10.00
	WHERE OrderLineID = 3;  --initial unitprice is 13.00

	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;
--Note transaction is still open
--Now go to Connection 2
--------
--------
--Returning from connection 2
ROLLBACK TRAN;

/*****************************************************/
-----Example 2  Connection 1--------------------------
 -----READ COMMITTED ISOLATION LEVEL---
 --Open a transaction and update the price of an item
 --Keep transaction open
 BEGIN TRAN;
	UPDATE Examples.IsoLevels
	SET UnitPrice +=10.00
	WHERE OrderLineID = 3;  --initial unitprice is 13.00

	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;
--Note transaction is still open
--Now go to Connection 2
--------
--------
--Returning from connection 2
COMMIT TRAN;

--Go back to connection 2 

--Cleanup changes from Example 2
UPDATE Examples.IsoLevels
	SET UnitPrice = 13.00
	WHERE OrderLineID = 3;

/**********************************************************/
/**********************************************************/
/*****************************************************/
-----Example 3  Connection 1--------------------------
 ----REPEATABLE READ ISOLATION LEVEL---
 --Open a transaction and update the price of an item
 --Keep transaction open
 SET TRANSACTION ISOLATION LEVEL REPEATABLE READ;
 
 BEGIN TRAN;

	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;

--Shared lock still held------
--Now go to connection 2-----
	
--Returning from connection 2
--This second read has the same value as the 
--first read in connection 1
	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;

--After commit the update from connection 2 can go through
--Data is updated
COMMIT TRAN;

	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;

--Cleanup changes from Example 3
	UPDATE Examples.IsoLevels
	SET UnitPrice = 13.00
	WHERE OrderLineID = 3;

/**********************************************************/
/**********************************************************/
/**********************************************************/
/*****************************************************/
-----Example 4  Connection 1--------------------------
-----SERIALIZABLE ISOLATION LEVEL---
--This example shows that serializable isolation level
--prevents phantom reads------------------------------

SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;

BEGIN TRAN;
	SELECT [OrderLineID]
      ,[OrderID]
      ,[StockItemID]
      ,[Quantity]
      ,[UnitPrice]
	FROM Examples.IsoLevels
	WHERE OrderID = 4;

--Now go to connection 2--------
--------------------------------
--Returning from connection 2, run the following--
--The output has not changed - no phantoms--------
SELECT [OrderLineID]
      ,[OrderID]
      ,[StockItemID]
      ,[Quantity]
      ,[UnitPrice]
	FROM Examples.IsoLevels
	WHERE OrderID = 4;

COMMIT TRAN;

--At this point the shared key range lock is released
--The modification in connection 2 can now proceed and
--insert the new row---------------------------------

SELECT [OrderLineID]
      ,[OrderID]
      ,[StockItemID]
      ,Description
	  ,[Quantity]
      ,[UnitPrice]
	FROM Examples.IsoLevels
	WHERE OrderID = 4;


--Cleanup changes from example 4----
DELETE FROM Examples.IsoLevels
WHERE OrderlineID > 40;

--Reset the isolation level-----------
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

/**********************************************************/
/**********************************************************/
/**********************************************************/
/**********************************************************/
/*****************************************************/
-----Example 5  Connection 1--------------------------
----SNAPSHOT ISOLATION LEVEL--------------------------

ALTER DATABASE WideWorldImporters SET ALLOW_SNAPSHOT_ISOLATION ON;

--Open a transaction and update the price of an item
 --Keep transaction open
 BEGIN TRAN;
	UPDATE Examples.IsoLevels
	SET UnitPrice +=10.00
	WHERE OrderLineID = 3;  --initial unitprice is 13.00

	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;
--Note transaction is still open
--Now go to Connection 2
--------
--------
--Returning from connection 2

COMMIT TRAN;

--Chack status of update
	SELECT OrderLineID, UnitPrice
	FROM Examples.IsoLevels
	WHERE OrderLineID = 3;

--Now go to Connection 2 again
--------

--Returning from connection 2---
--A cleanup thread which runs every minute 
--will remove the original data from tempdb
--so it will not fill up------------------

--Cleanup changes from Example 5
	UPDATE Examples.IsoLevels
	SET UnitPrice = 13.00
	WHERE OrderLineID = 3;


ALTER DATABASE WideWorldImporters
SET ALLOW_SNAPSHOT_ISOLATION OFF;


SET TRANSACTION ISOLATION LEVEL READ COMMITTED;